import os
from flask import Flask

def create_app(conf_name=None):

    app = Flask(__name__, instance_relative_config=True)

    # TODO
    def update_config(app):
        """Update app's config"""

        from .config import config
        if conf_name is not None:
            app.config.from_object(config[conf_name])
        else:
            app.config.from_object(config["default"])

        pass

    # TODO
    def configure_logging(app):
        """Update logger's configurations"""

        pass


    def register_blueprints(app):
        """Configures blueprint.

        e.g.
        from .apps.demo import bp
        bp.register(self)
        """

        from .apps.demo import bp
        app.register_blueprint(bp)

        pass


    def configure_extensions(app):
        """Configures the extensions.

        e.g.
        mail.init_app(app)

        """

        pass

    def configure_handlers(app):
        """Configures the handlers or middleware

        """

        # from .handlers import error_handler
        # app.register_error_handler(Exception, error_handler)

        pass

    # ensure the instance folder exists
    try:
        os.makedirs(app.instance_path)
        # os.makedirs(os.path.join(self.instance_path, "logs"))
    except OSError:
        pass

    # Initialization flask app
    update_config(app)
    configure_logging(app)
    configure_handlers(app)
    configure_extensions(app)
    register_blueprints(app)

    return app









import os
import sys
import click
import shutil
import subprocess


@click.group(
    help="""\
Flask-Wow
A simple CLI Generator for flask apps.

\b
  {prefix}flask-wow startproject newproj
""".format(
        # cmd="export" if os.name == "posix" else "set",
        prefix="$ " if os.name == "posix" else "> ",
    ))
def cli():
    pass


@cli.command('startproject')
@click.argument('proj_name', nargs=1)
@click.option('-v', '--venv', is_flag=True)
def startproject_command(proj_name, venv):

    click.echo('Creating project with name "{}"...'.format(proj_name))

    proj_source = os.path.join(
        os.path.dirname(__file__),
        'templates',
        'project_tmp',
        'proj_name')

    proj_dist = os.path.join(os.getcwd(), proj_name)

    app_source = os.path.join(
        os.path.dirname(__file__),
        'templates',
        'app_tmp',
        'app_name')

    app_dist = os.path.join(proj_dist, 'apps', 'demo')

    if not dir_check(proj_dist):
        click.echo(
            'The directory {} was not empty, Can\'t create project.'.format(proj_name))
        return None

    click.echo('Copying files to "{}" ...'.format(proj_dist))
    shutil.copytree(proj_source, proj_dist,
                    ignore=lambda src, names: [name for name in names if name == '__pycache__'])

    click.echo(
        'Copying demo files from {} to {}...'.format(
            app_source, app_dist))
    shutil.copytree(app_source, app_dist,
                    ignore=lambda src, names: [name for name in names if name == '__pycache__'])

    ch_name(proj_dist, project_name=proj_name)

    if venv:
        create_venv(proj_name)

    click.echo('Creating project at {} has succeeded!'.format(proj_dist))


@cli.command('startapp')
@click.argument('app_name', nargs=1)
def startapp_command(app_name):

    click.echo('Creating app with name "{}"...'.format(app_name))

    app_source = os.path.join(
        os.path.dirname(__file__),
        'templates',
        'app_tmp',
        'app_name')
    app_dist = os.path.join(os.getcwd(), 'apps', app_name)

    shutil.copytree(app_source, app_dist,
                    ignore=lambda src, names: [name for name in names if name == '__pycache__'])

    click.echo('new app "{}" has been successfully created！'.format(app_name))

    pass


def dir_check(dist_dir):
    if not os.path.exists(dist_dir) or not os.listdir(
            dist_dir):  # dir not exists
        return True
    return False


def ch_name(root, app_name=None, project_name=None):

    for fl in os.walk(root):
        dir, sub_dir, files = fl
        for file in files:
            if not file.endswith('.py'):
                continue
            fp = os.path.join(dir, file)
            with open(fp, 'r', encoding='utf-8') as f:
                content = f.read()
                if project_name:
                    content = content.replace(
                        "{% project_name %}", project_name)
                if app_name:
                    content = content.replace("{% app_name %}", app_name)
                with open(f'{fp}.bak', 'w', encoding='utf-8') as f:
                    f.write(content)

            os.remove(fp)
            os.rename(f'{fp}.bak', fp)


def create_venv(project_name):

    process = subprocess.Popen(
        'cd {} && python -m venv venv'.format(os.getcwd()), shell=True, stdout=subprocess.PIPE)
    click.echo('Creating Python virtual environment...')
    process.wait()
    if process.returncode == 0:
        print('Virtual environment created.')
    else:
        print('Virtual environment creation failed: {}'.format(process.stdout))


def main(as_module=False):
    cli.main(args=sys.argv[1:],
             prog_name="python -m flask-wow" if as_module else None)


if __name__ == "__main__":
    main(as_module=True)



class BaseConfig(object):
    """Base config"""

    SECRET_KEY = 'basic'
    DEBUG = False

class ProductionConfig(BaseConfig):
    """Use production Config."""




class DevelopmentConfig(BaseConfig):
    """Use development Config."""

    DEBUG = True


config = {
 'development': DevelopmentConfig,
 'production': ProductionConfig,
 'default': DevelopmentConfig
}


"""
    flask-wow
    ~~~~~~~~~

    A simple CLI Generator to create flask app.

    :copyright: 2020 Cove
    :license: BSD 3-Clause License
"""

__version__ = '0.2.1'


# def demo():
#
#     if args.cmd == 'addapp':
#         print(f'Will create Flask app with name "{args.name}"')
#         dir_name = os.path.dirname(__file__)
#         source_dir = os.path.join(dir_name, 'templates', 'app_tmp')
#         dist_dir = os.path.join(os.getcwd(), args.name)
#         if dist_dir_check(dist_dir):
#             shutil.copytree(source_dir, dist_dir,
#                             ignore=lambda src, names: [name for name in names if name == '__pycache__'])
#             format_dir(dist_dir, args.name, args.name)
#             print(f'Create app at {dist_dir} Success!')
#         else:
#             print(f'The directory {dist_dir} was not empty, Can\'t create app.')



from setuptools import setup
from flask_wow import __version__


with open("README.md", "r", encoding='UTF-8') as fh:
    long_description = fh.read()


setup( 
    name="flask-wow",
    version=__version__,
    description="A simple CLI Generator for flask apps.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords='extension flask flask-extension flask-build-tool flask_now flask-now app-generator flask-app-generator app generator',
    url="https://github.com/ganquan881205/flask-wow.git",
    author="Cove",
    author_email="cove@happyeyez.com",
    license="BSD-3-Clause",
    license_files = "LICENSE.rst",

    packages=["flask_wow"],
    include_package_data=True,
    platforms="any",
    install_requires=[
        'flask>=1.0.2',
        'click>=7.0'
    ],
    scripts=[],
    entry_points={
        'console_scripts': [
            'flask-wow = flask_wow.cli:main'
        ]
    },
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3 :: Only',
        'Framework :: Flask',
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
    ],
    zip_safe=False,
    python_requires='>=3.6',
)
